@@ -20,36 +20,44 @@ For a usage example, see the `Doc2vec tutorial
 <https://github.com/RaRe-Technologies/gensim/blob/develop/docs/notebooks/doc2vec-lee.ipynb>`_.
 
 **Make sure you have a C compiler before installing Gensim, to use the optimized doc2vec routines** (70x speedup
-compared to plain NumPy implementation <https://rare-technologies.com/parallelizing-word2vec-in-python/>`_).
+compared to plain NumPy implementation, https://rare-technologies.com/parallelizing-word2vec-in-python/).
 
 
-Examples
---------
+Usage examples
+==============
 
-Initialize & train a model
+Initialize & train a model:
 
->>> from gensim.test.utils import common_texts
->>> from gensim.models.doc2vec import Doc2Vec, TaggedDocument
->>>
->>> documents = [TaggedDocument(doc, [i]) for i, doc in enumerate(common_texts)]
->>> model = Doc2Vec(documents, vector_size=5, window=2, min_count=1, workers=4)
+.. sourcecode:: pycon
 
-Persist a model to disk
+    >>> from gensim.test.utils import common_texts
+    >>> from gensim.models.doc2vec import Doc2Vec, TaggedDocument
+    >>>
+    >>> documents = [TaggedDocument(doc, [i]) for i, doc in enumerate(common_texts)]
+    >>> model = Doc2Vec(documents, vector_size=5, window=2, min_count=1, workers=4)
 
->>> from gensim.test.utils import get_tmpfile
->>>
->>> fname = get_tmpfile("my_doc2vec_model")
->>>
->>> model.save(fname)
->>> model = Doc2Vec.load(fname)  # you can continue training with the loaded model!
+Persist a model to disk:
 
-If you're finished training a model (=no more updates, only querying, reduce memory usage), you can do
+.. sourcecode:: pycon
 
->>> model.delete_temporary_training_data(keep_doctags_vectors=True, keep_inference=True)
+    >>> from gensim.test.utils import get_tmpfile
+    >>>
+    >>> fname = get_tmpfile("my_doc2vec_model")
+    >>>
+    >>> model.save(fname)
+    >>> model = Doc2Vec.load(fname)  # you can continue training with the loaded model!
 
-Infer vector for new document
+If you're finished training a model (=no more updates, only querying, reduce memory usage), you can do:
 
->>> vector = model.infer_vector(["system", "response"])
+.. sourcecode:: pycon
+
+    >>> model.delete_temporary_training_data(keep_doctags_vectors=True, keep_inference=True)
+
+Infer vector for a new document:
+
+.. sourcecode:: pycon
+
+    >>> vector = model.infer_vector(["system", "response"])
 
 """
 
@@ -73,12 +81,12 @@ from gensim.utils import call_on_class_only
 from gensim import utils, matutils  # utility fnc for pickling, common scipy operations etc
 from gensim.models.word2vec import Word2VecKeyedVectors, Word2VecVocab, Word2VecTrainables, train_cbow_pair,\
     train_sg_pair, train_batch_sg
-from six.moves import xrange
+from six.moves import range
 from six import string_types, integer_types, itervalues
 from gensim.models.base_any2vec import BaseWordEmbeddingsModel
 from gensim.models.keyedvectors import Doc2VecKeyedVectors
 from types import GeneratorType
-from gensim.utils import deprecated
+from gensim.utils import deprecated, smart_open
 
 logger = logging.getLogger(__name__)
 
@@ -219,8 +227,8 @@ except ImportError:
         if doctag_locks is None:
             doctag_locks = model.docvecs.doctag_syn0_lockf
 
-        word_vocabs = [model.wv.vocab[w] for w in doc_words if w in model.wv.vocab and
-                       model.wv.vocab[w].sample_int > model.random.rand() * 2 ** 32]
+        word_vocabs = [model.wv.vocab[w] for w in doc_words if w in model.wv.vocab
+                       and model.wv.vocab[w].sample_int > model.random.rand() * 2 ** 32]
 
         for pos, word in enumerate(word_vocabs):
             reduced_window = model.random.randint(model.window)  # `b` in the original doc2vec code
@@ -306,8 +314,8 @@ except ImportError:
         if doctag_locks is None:
             doctag_locks = model.docvecs.doctag_syn0_lockf
 
-        word_vocabs = [model.wv.vocab[w] for w in doc_words if w in model.wv.vocab and
-                       model.wv.vocab[w].sample_int > model.random.rand() * 2 ** 32]
+        word_vocabs = [model.wv.vocab[w] for w in doc_words if w in model.wv.vocab
+                       and model.wv.vocab[w].sample_int > model.random.rand() * 2 ** 32]
         doctag_len = len(doctag_indexes)
         if doctag_len != model.dm_tag_count:
             return 0  # skip doc without expected number of doctag(s) (TODO: warn/pad?)
@@ -345,6 +353,36 @@ except ImportError:
         return len(padded_document_indexes) - pre_pad_count - post_pad_count
 
 
+try:
+    from gensim.models.doc2vec_corpusfile import (
+        d2v_train_epoch_dbow,
+        d2v_train_epoch_dm_concat,
+        d2v_train_epoch_dm,
+        CORPUSFILE_VERSION
+    )
+except ImportError:
+    # corpusfile doc2vec is not supported
+    CORPUSFILE_VERSION = -1
+
+    def d2v_train_epoch_dbow(model, corpus_file, offset, start_doctag, _cython_vocab, _cur_epoch, _expected_examples,
+                             _expected_words, work, _neu1, docvecs_count, word_vectors=None, word_locks=None,
+                             train_words=False, learn_doctags=True, learn_words=True, learn_hidden=True,
+                             doctag_vectors=None, doctag_locks=None):
+        raise NotImplementedError("Training with corpus_file argument is not supported.")
+
+    def d2v_train_epoch_dm_concat(model, corpus_file, offset, start_doctag, _cython_vocab, _cur_epoch,
+                                  _expected_examples, _expected_words, work, _neu1, docvecs_count, word_vectors=None,
+                                  word_locks=None, learn_doctags=True, learn_words=True, learn_hidden=True,
+                                  doctag_vectors=None, doctag_locks=None):
+        raise NotImplementedError("Training with corpus_file argument is not supported.")
+
+    def d2v_train_epoch_dm(model, corpus_file, offset, start_doctag, _cython_vocab, _cur_epoch, _expected_examples,
+                           _expected_words, work, _neu1, docvecs_count, word_vectors=None, word_locks=None,
+                           learn_doctags=True, learn_words=True, learn_hidden=True, doctag_vectors=None,
+                           doctag_locks=None):
+        raise NotImplementedError("Training with corpus_file argument is not supported.")
+
+
 class TaggedDocument(namedtuple('TaggedDocument', 'words tags')):
     """Represents a document along with a tag, input document format for :class:`~gensim.models.doc2vec.Doc2Vec`.
 
@@ -435,8 +473,9 @@ class Doc2Vec(BaseWordEmbeddingsModel):
         includes not only the word vectors of each word in the context, but also the paragraph vector.
 
     """
-    def __init__(self, documents=None, dm_mean=None, dm=1, dbow_words=0, dm_concat=0, dm_tag_count=1,
-                 docvecs=None, docvecs_mapfile=None, comment=None, trim_rule=None, callbacks=(), **kwargs):
+    def __init__(self, documents=None, corpus_file=None, dm_mean=None, dm=1, dbow_words=0, dm_concat=0,
+                 dm_tag_count=1, docvecs=None, docvecs_mapfile=None, comment=None, trim_rule=None, callbacks=(),
+                 **kwargs):
         """
 
         Parameters
@@ -445,10 +484,15 @@ class Doc2Vec(BaseWordEmbeddingsModel):
             Input corpus, can be simply a list of elements, but for larger corpora,consider an iterable that streams
             the documents directly from disk/network. If you don't supply `documents`, the model is
             left uninitialized -- use if you plan to initialize it in some other way.
+        corpus_file : str, optional
+            Path to a corpus file in :class:`~gensim.models.word2vec.LineSentence` format.
+            You may use this argument instead of `sentences` to get performance boost. Only one of `sentences` or
+            `corpus_file` arguments need to be passed (or none of them). Documents' tags are assigned automatically
+            and are equal to line number, as in :class:`~gensim.models.doc2vec.TaggedLineDocument`.
         dm : {1,0}, optional
             Defines the training algorithm. If `dm=1`, 'distributed memory' (PV-DM) is used.
             Otherwise, `distributed bag of words` (PV-DBOW) is employed.
-        size : int, optional
+        vector_size : int, optional
             Dimensionality of the feature vectors.
         window : int, optional
             The maximum distance between the current and predicted word within a sentence.
@@ -474,7 +518,7 @@ class Doc2Vec(BaseWordEmbeddingsModel):
             useful range is (0, 1e-5).
         workers : int, optional
             Use these many worker threads to train the model (=faster training with multicore machines).
-        iter : int, optional
+        epochs : int, optional
             Number of iterations (epochs) over the corpus.
         hs : {1,0}, optional
             If 1, hierarchical softmax will be used for model training.
@@ -566,13 +610,18 @@ class Doc2Vec(BaseWordEmbeddingsModel):
         self.docvecs = docvecs or Doc2VecKeyedVectors(self.vector_size, docvecs_mapfile)
 
         self.comment = comment
-        if documents is not None:
-            if isinstance(documents, GeneratorType):
+
+        if documents is not None or corpus_file is not None:
+            self._check_input_data_sanity(data_iterable=documents, corpus_file=corpus_file)
+            if corpus_file is not None and not isinstance(corpus_file, string_types):
+                raise TypeError("You must pass string as the corpus_file argument.")
+            elif isinstance(documents, GeneratorType):
                 raise TypeError("You can't pass a generator as the documents argument. Try an iterator.")
-            self.build_vocab(documents, trim_rule=trim_rule)
+            self.build_vocab(documents=documents, corpus_file=corpus_file, trim_rule=trim_rule)
             self.train(
-                documents, total_examples=self.corpus_count, epochs=self.epochs,
-                start_alpha=self.alpha, end_alpha=self.min_alpha, callbacks=callbacks)
+                documents=documents, corpus_file=corpus_file, total_examples=self.corpus_count,
+                total_words=self.corpus_total_words, epochs=self.epochs, start_alpha=self.alpha,
+                end_alpha=self.min_alpha, callbacks=callbacks)
 
     @property
     def dm(self):
@@ -620,6 +669,33 @@ class Doc2Vec(BaseWordEmbeddingsModel):
         self.docvecs.offset2doctag = other_model.docvecs.offset2doctag
         self.trainables.reset_weights(self.hs, self.negative, self.wv, self.docvecs)
 
+    def _do_train_epoch(self, corpus_file, thread_id, offset, cython_vocab, thread_private_mem, cur_epoch,
+                        total_examples=None, total_words=None, offsets=None, start_doctags=None, **kwargs):
+        work, neu1 = thread_private_mem
+        doctag_vectors = self.docvecs.vectors_docs
+        doctag_locks = self.trainables.vectors_docs_lockf
+
+        offset = offsets[thread_id]
+        start_doctag = start_doctags[thread_id]
+
+        if self.sg:
+            examples, tally, raw_tally = d2v_train_epoch_dbow(
+                self, corpus_file, offset, start_doctag, cython_vocab, cur_epoch,
+                total_examples, total_words, work, neu1, self.docvecs.count,
+                doctag_vectors=doctag_vectors, doctag_locks=doctag_locks, train_words=self.dbow_words)
+        elif self.dm_concat:
+            examples, tally, raw_tally = d2v_train_epoch_dm_concat(
+                self, corpus_file, offset, start_doctag, cython_vocab, cur_epoch,
+                total_examples, total_words, work, neu1, self.docvecs.count,
+                doctag_vectors=doctag_vectors, doctag_locks=doctag_locks)
+        else:
+            examples, tally, raw_tally = d2v_train_epoch_dm(
+                self, corpus_file, offset, start_doctag, cython_vocab, cur_epoch,
+                total_examples, total_words, work, neu1, self.docvecs.count,
+                doctag_vectors=doctag_vectors, doctag_locks=doctag_locks)
+
+        return examples, tally, raw_tally
+
     def _do_train_job(self, job, alpha, inits):
         """Train model using `job` data.
 
@@ -661,7 +737,7 @@ class Doc2Vec(BaseWordEmbeddingsModel):
                 )
         return tally, self._raw_word_count(job)
 
-    def train(self, documents, total_examples=None, total_words=None,
+    def train(self, documents=None, corpus_file=None, total_examples=None, total_words=None,
               epochs=None, start_alpha=None, end_alpha=None,
               word_count=0, queue_factor=2, report_delay=1.0, callbacks=()):
         """Update the model's neural weights.
@@ -679,10 +755,15 @@ class Doc2Vec(BaseWordEmbeddingsModel):
 
         Parameters
         ----------
-        documents : iterable of list of :class:`~gensim.models.doc2vec.TaggedDocument`
+        documents : iterable of list of :class:`~gensim.models.doc2vec.TaggedDocument`, optional
             Can be simply a list of elements, but for larger corpora,consider an iterable that streams
             the documents directly from disk/network. If you don't supply `documents`, the model is
             left uninitialized -- use if you plan to initialize it in some other way.
+        corpus_file : str, optional
+            Path to a corpus file in :class:`~gensim.models.word2vec.LineSentence` format.
+            You may use this argument instead of `sentences` to get performance boost. Only one of `sentences` or
+            `corpus_file` arguments need to be passed (not both of them). Documents' tags are assigned automatically
+            and are equal to line number, as in :class:`~gensim.models.doc2vec.TaggedLineDocument`.
         total_examples : int, optional
             Count of sentences.
         total_words : int, optional
@@ -711,10 +792,61 @@ class Doc2Vec(BaseWordEmbeddingsModel):
             List of callbacks that need to be executed/run at specific stages during training.
 
         """
+        kwargs = {}
+        if corpus_file is not None:
+            # Calculate offsets for each worker along with initial doctags (doctag ~ document/line number in a file)
+            offsets, start_doctags = self._get_offsets_and_start_doctags_for_corpusfile(corpus_file, self.workers)
+            kwargs['offsets'] = offsets
+            kwargs['start_doctags'] = start_doctags
+
         super(Doc2Vec, self).train(
-            documents, total_examples=total_examples, total_words=total_words,
+            sentences=documents, corpus_file=corpus_file, total_examples=total_examples, total_words=total_words,
             epochs=epochs, start_alpha=start_alpha, end_alpha=end_alpha, word_count=word_count,
-            queue_factor=queue_factor, report_delay=report_delay, callbacks=callbacks)
+            queue_factor=queue_factor, report_delay=report_delay, callbacks=callbacks, **kwargs)
+
+    @classmethod
+    def _get_offsets_and_start_doctags_for_corpusfile(cls, corpus_file, workers):
+        """Get offset and initial document tag in a corpus_file for each worker.
+
+        Firstly, approximate offsets are calculated based on number of workers and corpus_file size.
+        Secondly, for each approximate offset we find the maximum offset which points to the beginning of line and
+        less than approximate offset.
+
+        Parameters
+        ----------
+        corpus_file : str
+            Path to a corpus file in :class:`~gensim.models.word2vec.LineSentence` format.
+        workers : int
+            Number of workers.
+
+        Returns
+        -------
+        list of int, list of int
+            Lists with offsets and document tags with length = number of workers.
+        """
+        corpus_file_size = os.path.getsize(corpus_file)
+        approx_offsets = [int(corpus_file_size // workers * i) for i in range(workers)]
+        offsets = []
+        start_doctags = []
+
+        with smart_open(corpus_file, mode='rb') as fin:
+            curr_offset_idx = 0
+            prev_filepos = 0
+
+            for line_no, line in enumerate(fin):
+                if curr_offset_idx == len(approx_offsets):
+                    break
+
+                curr_filepos = prev_filepos + len(line)
+                while curr_offset_idx != len(approx_offsets) and approx_offsets[curr_offset_idx] < curr_filepos:
+                    offsets.append(prev_filepos)
+                    start_doctags.append(line_no)
+
+                    curr_offset_idx += 1
+
+                prev_filepos = curr_filepos
+
+        return offsets, start_doctags
 
     def _raw_word_count(self, job):
         """Get the number of words in a given job.
@@ -911,7 +1043,7 @@ class Doc2Vec(BaseWordEmbeddingsModel):
         fvocab : str, optional
             Optional file path used to save the vocabulary.
         binary : bool, optional
-            If True, the data wil be saved in binary word2vec format, otherwise - will be saved in plain text.
+            If True, the data will be saved in binary word2vec format, otherwise - will be saved in plain text.
 
         """
         total_vec = len(self.wv.vocab) + len(self.docvecs)
@@ -997,15 +1129,21 @@ class Doc2Vec(BaseWordEmbeddingsModel):
         report['doctag_syn0'] = self.docvecs.count * self.vector_size * dtype(REAL).itemsize
         return super(Doc2Vec, self).estimate_memory(vocab_size, report=report)
 
-    def build_vocab(self, documents, update=False, progress_per=10000, keep_raw_vocab=False, trim_rule=None, **kwargs):
+    def build_vocab(self, documents=None, corpus_file=None, update=False, progress_per=10000, keep_raw_vocab=False,
+                    trim_rule=None, **kwargs):
         """Build vocabulary from a sequence of sentences (can be a once-only generator stream).
 
         Parameters
         ----------
-        documents : iterable of list of :class:`~gensim.models.doc2vec.TaggedDocument`
+        documents : iterable of list of :class:`~gensim.models.doc2vec.TaggedDocument`, optional
             Can be simply a list of :class:`~gensim.models.doc2vec.TaggedDocument` elements, but for larger corpora,
             consider an iterable that streams the documents directly from disk/network.
             See :class:`~gensim.models.doc2vec.TaggedBrownCorpus` or :class:`~gensim.models.doc2vec.TaggedLineDocument`
+        corpus_file : str, optional
+            Path to a corpus file in :class:`~gensim.models.word2vec.LineSentence` format.
+            You may use this argument instead of `sentences` to get performance boost. Only one of `sentences` or
+            `corpus_file` arguments need to be passed (not both of them). Documents' tags are assigned automatically
+            and are equal to a line number, as in :class:`~gensim.models.doc2vec.TaggedLineDocument`.
         update : bool
             If true, the new words in `sentences` will be added to model's vocab.
         progress_per : int
@@ -1031,8 +1169,11 @@ class Doc2Vec(BaseWordEmbeddingsModel):
 
         """
         total_words, corpus_count = self.vocabulary.scan_vocab(
-            documents, self.docvecs, progress_per=progress_per, trim_rule=trim_rule)
+            documents=documents, corpus_file=corpus_file, docvecs=self.docvecs,
+            progress_per=progress_per, trim_rule=trim_rule
+        )
         self.corpus_count = corpus_count
+        self.corpus_total_words = total_words
         report_values = self.vocabulary.prepare_vocab(
             self.hs, self.negative, self.wv, update=update, keep_raw_vocab=keep_raw_vocab, trim_rule=trim_rule,
             **kwargs)
@@ -1095,6 +1236,19 @@ class Doc2Vec(BaseWordEmbeddingsModel):
             self.hs, self.negative, self.wv, self.docvecs, update=update)
 
 
+def _note_doctag(key, document_length, docvecs):
+    """Note a document tag during initial corpus scan, for structure sizing."""
+    if isinstance(key, integer_types + (integer,)):
+        docvecs.max_rawint = max(docvecs.max_rawint, key)
+    else:
+        if key in docvecs.doctags:
+            docvecs.doctags[key] = docvecs.doctags[key].repeat(document_length)
+        else:
+            docvecs.doctags[key] = Doctag(len(docvecs.offset2doctag), document_length, 1)
+            docvecs.offset2doctag.append(key)
+    docvecs.count = docvecs.max_rawint + 1 + len(docvecs.offset2doctag)
+
+
 class Doc2VecVocab(Word2VecVocab):
     """Vocabulary used by :class:`~gensim.models.doc2vec.Doc2Vec`.
 
@@ -1111,7 +1265,7 @@ class Doc2VecVocab(Word2VecVocab):
             if there are more unique words than this, then prune the infrequent ones.
             Every 10 million word types need about 1GB of RAM, set to `None` for no limit.
         min_count : int
-            Words with frequency lower than this limit will be discarded form the vocabulary.
+            Words with frequency lower than this limit will be discarded from the vocabulary.
         sample : float, optional
             The threshold for configuring which higher-frequency words are randomly downsampled,
             useful range is (0, 1e-5).
@@ -1132,38 +1286,7 @@ class Doc2VecVocab(Word2VecVocab):
             max_vocab_size=max_vocab_size, min_count=min_count, sample=sample,
             sorted_vocab=sorted_vocab, null_word=null_word, ns_exponent=ns_exponent)
 
-    def scan_vocab(self, documents, docvecs, progress_per=10000, trim_rule=None):
-        """Create the models Vocabulary: A mapping from unique words in the corpus to their frequency count.
-
-        Parameters
-        ----------
-        documents : iterable of :class:`~gensim.models.doc2vec.TaggedDocument`
-            The tagged documents used to create the vocabulary. Their tags can be either str tokens or ints (faster).
-        docvecs : list of :class:`~gensim.models.keyedvectors.Doc2VecKeyedVectors`
-            The vector representations of the documents in our corpus. Each of them has a size == `vector_size`.
-        progress_per : int
-            Progress will be logged every `progress_per` documents.
-        trim_rule : function, optional
-            Vocabulary trimming rule, specifies whether certain words should remain in the vocabulary,
-            be trimmed away, or handled using the default (discard if word count < min_count).
-            Can be None (min_count will be used, look to :func:`~gensim.utils.keep_vocab_item`),
-            or a callable that accepts parameters (word, count, min_count) and returns either
-            :attr:`gensim.utils.RULE_DISCARD`, :attr:`gensim.utils.RULE_KEEP` or :attr:`gensim.utils.RULE_DEFAULT`.
-            The rule, if given, is only used to prune vocabulary during
-            :meth:`~gensim.models.doc2vec.Doc2Vec.build_vocab` and is not stored as part of the model.
-
-            The input parameters are of the following types:
-                * `word` (str) - the word we are examining
-                * `count` (int) - the word's frequency count in the corpus
-                * `min_count` (int) - the minimum count threshold.
-
-        Returns
-        -------
-        (int, int)
-            Tuple of (Total words in the corpus, number of documents)
-
-        """
-        logger.info("collecting all words and their counts")
+    def _scan_vocab(self, documents, docvecs, progress_per, trim_rule):
         document_no = -1
         total_words = 0
         min_reduce = 1
@@ -1191,7 +1314,7 @@ class Doc2VecVocab(Word2VecVocab):
             document_length = len(document.words)
 
             for tag in document.tags:
-                self.note_doctag(tag, document_no, document_length, docvecs)
+                _note_doctag(tag, document_length, docvecs)
 
             for word in document.words:
                 vocab[word] += 1
@@ -1201,38 +1324,57 @@ class Doc2VecVocab(Word2VecVocab):
                 utils.prune_vocab(vocab, min_reduce, trim_rule=trim_rule)
                 min_reduce += 1
 
-        logger.info(
-            "collected %i word types and %i unique tags from a corpus of %i examples and %i words",
-            len(vocab), docvecs.count, document_no + 1, total_words
-        )
         corpus_count = document_no + 1
         self.raw_vocab = vocab
         return total_words, corpus_count
 
-    def note_doctag(self, key, document_no, document_length, docvecs):
-        """Note a document tag during initial corpus scan, for correctly setting the keyedvectors size.
+    def scan_vocab(self, documents=None, corpus_file=None, docvecs=None, progress_per=10000, trim_rule=None):
+        """Create the models Vocabulary: A mapping from unique words in the corpus to their frequency count.
 
         Parameters
         ----------
-        key : {int, str}
-            The tag to be noted.
-        document_no : int
-            The document's index in `docvecs`. Unused.
-        document_length : int
-            The document's length in words.
+        documents : iterable of :class:`~gensim.models.doc2vec.TaggedDocument`, optional
+            The tagged documents used to create the vocabulary. Their tags can be either str tokens or ints (faster).
+        corpus_file : str, optional
+            Path to a corpus file in :class:`~gensim.models.word2vec.LineSentence` format.
+            You may use this argument instead of `sentences` to get performance boost. Only one of `sentences` or
+            `corpus_file` arguments need to be passed (not both of them).
         docvecs : list of :class:`~gensim.models.keyedvectors.Doc2VecKeyedVectors`
-            Vector representations of the documents in the corpus. Each vector has size == `vector_size`
+            The vector representations of the documents in our corpus. Each of them has a size == `vector_size`.
+        progress_per : int
+            Progress will be logged every `progress_per` documents.
+        trim_rule : function, optional
+            Vocabulary trimming rule, specifies whether certain words should remain in the vocabulary,
+            be trimmed away, or handled using the default (discard if word count < min_count).
+            Can be None (min_count will be used, look to :func:`~gensim.utils.keep_vocab_item`),
+            or a callable that accepts parameters (word, count, min_count) and returns either
+            :attr:`gensim.utils.RULE_DISCARD`, :attr:`gensim.utils.RULE_KEEP` or :attr:`gensim.utils.RULE_DEFAULT`.
+            The rule, if given, is only used to prune vocabulary during
+            :meth:`~gensim.models.doc2vec.Doc2Vec.build_vocab` and is not stored as part of the model.
+
+            The input parameters are of the following types:
+                * `word` (str) - the word we are examining
+                * `count` (int) - the word's frequency count in the corpus
+                * `min_count` (int) - the minimum count threshold.
+
+        Returns
+        -------
+        (int, int)
+            Tuple of (Total words in the corpus, number of documents)
 
         """
-        if isinstance(key, integer_types + (integer,)):
-            docvecs.max_rawint = max(docvecs.max_rawint, key)
-        else:
-            if key in docvecs.doctags:
-                docvecs.doctags[key] = docvecs.doctags[key].repeat(document_length)
-            else:
-                docvecs.doctags[key] = Doctag(len(docvecs.offset2doctag), document_length, 1)
-                docvecs.offset2doctag.append(key)
-        docvecs.count = docvecs.max_rawint + 1 + len(docvecs.offset2doctag)
+        logger.info("collecting all words and their counts")
+        if corpus_file is not None:
+            documents = TaggedLineDocument(corpus_file)
+
+        total_words, corpus_count = self._scan_vocab(documents, docvecs, progress_per, trim_rule)
+
+        logger.info(
+            "collected %i word types and %i unique tags from a corpus of %i examples and %i words",
+            len(self.raw_vocab), docvecs.count, corpus_count, total_words
+        )
+
+        return total_words, corpus_count
 
     def indexed_doctags(self, doctag_tokens, docvecs):
         """Get the indexes and backing-arrays used in training examples.
@@ -1311,7 +1453,7 @@ class Doc2VecTrainables(Word2VecTrainables):
             docvecs.vectors_docs = empty((length, docvecs.vector_size), dtype=REAL)
             self.vectors_docs_lockf = ones((length,), dtype=REAL)  # zeros suppress learning
 
-        for i in xrange(length):
+        for i in range(length):
             # construct deterministic seed from index AND model seed
             seed = "%d %s" % (
                 self.seed, Doc2VecKeyedVectors._index_to_doctag(i, docvecs.offset2doctag, docvecs.max_rawint))
@@ -1380,11 +1522,13 @@ class TaggedLineDocument(object):
 
         Examples
         --------
-        >>> from gensim.test.utils import datapath
-        >>> from gensim.models.doc2vec import TaggedLineDocument
-        >>>
-        >>> for document in TaggedLineDocument(datapath("head500.noblanks.cor")):
-        ...     pass
+        .. sourcecode:: pycon
+
+            >>> from gensim.test.utils import datapath
+            >>> from gensim.models.doc2vec import TaggedLineDocument
+            >>>
+            >>> for document in TaggedLineDocument(datapath("head500.noblanks.cor")):
+            ...     pass
 
         """
         self.source = source
