@@ -27,53 +27,64 @@ For a tutorial on Gensim word2vec, with an interactive web app trained on Google
 visit https://rare-technologies.com/word2vec-tutorial/.
 
 **Make sure you have a C compiler before installing Gensim, to use the optimized word2vec routines**
-(70x speedup compared to plain NumPy implementation, https://rare-technologies.com/parallelizing-word2vec-in-python/.
+(70x speedup compared to plain NumPy implementation, https://rare-technologies.com/parallelizing-word2vec-in-python/).
 
 Usage examples
 ==============
 
-Initialize a model with e.g.
+Initialize a model with e.g.:
 
->>> from gensim.test.utils import common_texts, get_tmpfile
->>> from gensim.models import Word2Vec
->>>
->>> path = get_tmpfile("word2vec.model")
->>>
->>> model = Word2Vec(common_texts, size=100, window=5, min_count=1, workers=4)
->>> model.save("word2vec.model")
+.. sourcecode:: pycon
+
+    >>> from gensim.test.utils import common_texts, get_tmpfile
+    >>> from gensim.models import Word2Vec
+    >>>
+    >>> path = get_tmpfile("word2vec.model")
+    >>>
+    >>> model = Word2Vec(common_texts, size=100, window=5, min_count=1, workers=4)
+    >>> model.save("word2vec.model")
 
 The training is streamed, meaning `sentences` can be a generator, reading input data
 from disk on-the-fly, without loading the entire corpus into RAM.
 
-It also means you can continue training the model later
+It also means you can continue training the model later:
 
->>> model = Word2Vec.load("word2vec.model")
->>> model.train([["hello", "world"]], total_examples=1, epochs=1)
-(0, 2)
+.. sourcecode:: pycon
 
-The trained word vectors are stored in a :class:`~gensim.models.KeyedVectors` instance in `model.wv`:
+    >>> model = Word2Vec.load("word2vec.model")
+    >>> model.train([["hello", "world"]], total_examples=1, epochs=1)
+    (0, 2)
 
->>> vector = model.wv['computer']  # numpy vector of a word
+The trained word vectors are stored in a :class:`~gensim.models.keyedvectors.KeyedVectors` instance in `model.wv`:
+
+.. sourcecode:: pycon
+
+    >>> vector = model.wv['computer']  # numpy vector of a word
 
 The reason for separating the trained vectors into `KeyedVectors` is that if you don't
 need the full model state any more (don't need to continue training), the state can discarded,
 resulting in a much smaller and faster object that can be mmapped for lightning
-fast loading and sharing the vectors in RAM between processes::
+fast loading and sharing the vectors in RAM between processes:
 
->>> from gensim.models import KeyedVectors
->>>
->>> path = get_tmpfile("wordvectors.kv")
->>>
->>> model.wv.save(path)
->>> wv = KeyedVectors.load("model.wv", mmap='r')
->>> vector = wv['computer']  # numpy vector of a word
+.. sourcecode:: pycon
 
-Gensim can also load word vectors in the "word2vec C format", as this :class:`~gensim.models.KeyedVectors` instance::
+    >>> from gensim.models import KeyedVectors
+    >>>
+    >>> path = get_tmpfile("wordvectors.kv")
+    >>>
+    >>> model.wv.save(path)
+    >>> wv = KeyedVectors.load("model.wv", mmap='r')
+    >>> vector = wv['computer']  # numpy vector of a word
 
->>> from gensim.test.utils import datapath
->>>
->>> wv_from_text = KeyedVectors.load_word2vec_format(datapath('word2vec_pre_kv_c'), binary=False)  # C text format
->>> wv_from_bin = KeyedVectors.load_word2vec_format(datapath("euclidean_vectors.bin"), binary=True)  # C binary format
+Gensim can also load word vectors in the "word2vec C format", as a
+:class:`~gensim.models.keyedvectors.KeyedVectors` instance:
+
+.. sourcecode:: pycon
+
+    >>> from gensim.test.utils import datapath
+    >>>
+    >>> wv_from_text = KeyedVectors.load_word2vec_format(datapath('word2vec_pre_kv_c'), binary=False)  # C text format
+    >>> wv_from_bin = KeyedVectors.load_word2vec_format(datapath("euclidean_vectors.bin"), binary=True)  # C bin format
 
 It is impossible to continue training the vectors loaded from the C format because the hidden weights,
 vocabulary frequencies and the binary tree are missing. To continue training, you'll need the
@@ -84,10 +95,12 @@ You can perform various NLP word tasks with a trained model. Some of them
 are already built-in - you can see it in :mod:`gensim.models.keyedvectors`.
 
 If you're finished training a model (i.e. no more updates, only querying),
-you can switch to the :class:`~gensim.models.KeyedVectors` instance
+you can switch to the :class:`~gensim.models.keyedvectors.KeyedVectors` instance:
 
->>> word_vectors = model.wv
->>> del model
+.. sourcecode:: pycon
+
+    >>> word_vectors = model.wv
+    >>> del model
 
 to trim unneeded model state = use much less RAM and allow fast loading and memory sharing (mmap).
 
@@ -95,11 +108,13 @@ Note that there is a :mod:`gensim.models.phrases` module which lets you automati
 detect phrases longer than one word. Using phrases, you can learn a word2vec model
 where "words" are actually multiword expressions, such as `new_york_times` or `financial_crisis`:
 
->>> from gensim.test.utils import common_texts
->>> from gensim.models import Phrases
->>>
->>> bigram_transformer = Phrases(common_texts)
->>> model = Word2Vec(bigram_transformer[common_texts], min_count=1)
+.. sourcecode:: pycon
+
+    >>> from gensim.test.utils import common_texts
+    >>> from gensim.models import Phrases
+    >>>
+    >>> bigram_transformer = Phrases(common_texts)
+    >>> model = Word2Vec(bigram_transformer[common_texts], min_count=1)
 
 """
 
@@ -134,7 +149,7 @@ from scipy.special import expit
 from gensim import utils, matutils  # utility fnc for pickling, common scipy operations etc
 from gensim.utils import deprecated
 from six import iteritems, itervalues, string_types
-from six.moves import xrange
+from six.moves import range
 
 logger = logging.getLogger(__name__)
 
@@ -180,8 +195,8 @@ except ImportError:
         """
         result = 0
         for sentence in sentences:
-            word_vocabs = [model.wv.vocab[w] for w in sentence if w in model.wv.vocab and
-                           model.wv.vocab[w].sample_int > model.random.rand() * 2 ** 32]
+            word_vocabs = [model.wv.vocab[w] for w in sentence if w in model.wv.vocab
+                           and model.wv.vocab[w].sample_int > model.random.rand() * 2 ** 32]
             for pos, word in enumerate(word_vocabs):
                 reduced_window = model.random.randint(model.window)  # `b` in the original word2vec code
 
@@ -232,8 +247,8 @@ except ImportError:
         result = 0
         for sentence in sentences:
             word_vocabs = [
-                model.wv.vocab[w] for w in sentence if w in model.wv.vocab and
-                model.wv.vocab[w].sample_int > model.random.rand() * 2 ** 32
+                model.wv.vocab[w] for w in sentence if w in model.wv.vocab
+                and model.wv.vocab[w].sample_int > model.random.rand() * 2 ** 32
             ]
             for pos, word in enumerate(word_vocabs):
                 reduced_window = model.random.randint(model.window)  # `b` in the original word2vec code
@@ -332,6 +347,20 @@ except ImportError:
 
         return log_prob_sentence
 
+try:
+    from gensim.models.word2vec_corpusfile import train_epoch_sg, train_epoch_cbow, CORPUSFILE_VERSION
+except ImportError:
+    # file-based word2vec is not supported
+    CORPUSFILE_VERSION = -1
+
+    def train_epoch_sg(model, corpus_file, offset, _cython_vocab, _cur_epoch, _expected_examples, _expected_words,
+                       _work, _neu1, compute_loss):
+        raise RuntimeError("Training with corpus_file argument is not supported")
+
+    def train_epoch_cbow(model, corpus_file, offset, _cython_vocab, _cur_epoch, _expected_examples, _expected_words,
+                         _work, _neu1, compute_loss):
+        raise RuntimeError("Training with corpus_file argument is not supported")
+
 
 def train_sg_pair(model, word, context_index, alpha, learn_vectors=True, learn_hidden=True,
                   context_vectors=None, context_locks=None, compute_loss=False, is_ft=False):
@@ -614,7 +643,7 @@ class Word2Vec(BaseWordEmbeddingsModel):
         This object essentially contains the mapping between words and embeddings. After training, it can be used
         directly to query those embeddings in various ways. See the module level docstring for examples.
 
-    vocabulary : :class:'~gensim.models.word2vec.Word2VecVocab'
+    vocabulary : :class:`~gensim.models.word2vec.Word2VecVocab`
         This object represents the vocabulary (sometimes called Dictionary in gensim) of the model.
         Besides keeping track of all unique words, this object provides extra functionality, such as
         constructing a huffman tree (frequent words are closer to the root), or discarding extremely rare words.
@@ -626,7 +655,8 @@ class Word2Vec(BaseWordEmbeddingsModel):
         (which means that the size of the hidden layer is equal to the number of features `self.size`).
 
     """
-    def __init__(self, sentences=None, size=100, alpha=0.025, window=5, min_count=5,
+
+    def __init__(self, sentences=None, corpus_file=None, size=100, alpha=0.025, window=5, min_count=5,
                  max_vocab_size=None, sample=1e-3, seed=1, workers=3, min_alpha=0.0001,
                  sg=0, hs=0, negative=5, ns_exponent=0.75, cbow_mean=1, hashfxn=hash, iter=5, null_word=0,
                  trim_rule=None, sorted_vocab=1, batch_words=MAX_WORDS_IN_BATCH, compute_loss=False, callbacks=(),
@@ -644,6 +674,10 @@ class Word2Vec(BaseWordEmbeddingsModel):
             <https://rare-technologies.com/data-streaming-in-python-generators-iterators-iterables/>`_.
             If you don't supply `sentences`, the model is left uninitialized -- use if you plan to initialize it
             in some other way.
+        corpus_file : str, optional
+            Path to a corpus file in :class:`~gensim.models.word2vec.LineSentence` format.
+            You may use this argument instead of `sentences` to get performance boost. Only one of `sentences` or
+            `corpus_file` arguments need to be passed (or none of them).
         size : int, optional
             Dimensionality of the word vectors.
         window : int, optional
@@ -707,7 +741,6 @@ class Word2Vec(BaseWordEmbeddingsModel):
                 * `word` (str) - the word we are examining
                 * `count` (int) - the word's frequency count in the corpus
                 * `min_count` (int) - the minimum count threshold.
-
         sorted_vocab : {0, 1}, optional
             If 1, sort the vocabulary by descending frequency before assigning word indexes.
             See :meth:`~gensim.models.word2vec.Word2VecVocab.sort_vocab()`.
@@ -725,9 +758,11 @@ class Word2Vec(BaseWordEmbeddingsModel):
         --------
         Initialize and train a :class:`~gensim.models.word2vec.Word2Vec` model
 
-        >>> from gensim.models import Word2Vec
-        >>> sentences = [["cat", "say", "meow"], ["dog", "say", "woof"]]
-        >>> model = Word2Vec(sentences, min_count=1)
+        .. sourcecode:: pycon
+
+            >>> from gensim.models import Word2Vec
+            >>> sentences = [["cat", "say", "meow"], ["dog", "say", "woof"]]
+            >>> model = Word2Vec(sentences, min_count=1)
 
         """
         self.max_final_vocab = max_final_vocab
@@ -742,11 +777,24 @@ class Word2Vec(BaseWordEmbeddingsModel):
         self.trainables = Word2VecTrainables(seed=seed, vector_size=size, hashfxn=hashfxn)
 
         super(Word2Vec, self).__init__(
-            sentences=sentences, workers=workers, vector_size=size, epochs=iter, callbacks=callbacks,
-            batch_words=batch_words, trim_rule=trim_rule, sg=sg, alpha=alpha, window=window, seed=seed,
-            hs=hs, negative=negative, cbow_mean=cbow_mean, min_alpha=min_alpha, compute_loss=compute_loss,
+            sentences=sentences, corpus_file=corpus_file, workers=workers, vector_size=size, epochs=iter,
+            callbacks=callbacks, batch_words=batch_words, trim_rule=trim_rule, sg=sg, alpha=alpha, window=window,
+            seed=seed, hs=hs, negative=negative, cbow_mean=cbow_mean, min_alpha=min_alpha, compute_loss=compute_loss,
             fast_version=FAST_VERSION)
 
+    def _do_train_epoch(self, corpus_file, thread_id, offset, cython_vocab, thread_private_mem, cur_epoch,
+                        total_examples=None, total_words=None, **kwargs):
+        work, neu1 = thread_private_mem
+
+        if self.sg:
+            examples, tally, raw_tally = train_epoch_sg(self, corpus_file, offset, cython_vocab, cur_epoch,
+                                                        total_examples, total_words, work, neu1, self.compute_loss)
+        else:
+            examples, tally, raw_tally = train_epoch_cbow(self, corpus_file, offset, cython_vocab, cur_epoch,
+                                                          total_examples, total_words, work, neu1, self.compute_loss)
+
+        return examples, tally, raw_tally
+
     def _do_train_job(self, sentences, alpha, inits):
         """Train the model on a single batch of sentences.
 
@@ -782,7 +830,7 @@ class Word2Vec(BaseWordEmbeddingsModel):
             self.compute_loss = kwargs['compute_loss']
         self.running_training_loss = 0
 
-    def train(self, sentences, total_examples=None, total_words=None,
+    def train(self, sentences=None, corpus_file=None, total_examples=None, total_words=None,
               epochs=None, start_alpha=None, end_alpha=None, word_count=0,
               queue_factor=2, report_delay=1.0, compute_loss=False, callbacks=()):
         """Update the model's neural weights from a sequence of sentences.
@@ -810,11 +858,15 @@ class Word2Vec(BaseWordEmbeddingsModel):
             or :class:`~gensim.models.word2vec.LineSentence` in :mod:`~gensim.models.word2vec` module for such examples.
             See also the `tutorial on data streaming in Python
             <https://rare-technologies.com/data-streaming-in-python-generators-iterators-iterables/>`_.
-        total_examples : int, optional
-            Count of sentences. Used to decay the `alpha` learning rate.
-        total_words : int, optional
-            Count of raw words in sentences. Used to decay the `alpha` learning rate.
-        epochs : int, optional
+        corpus_file : str, optional
+            Path to a corpus file in :class:`~gensim.models.word2vec.LineSentence` format.
+            You may use this argument instead of `sentences` to get performance boost. Only one of `sentences` or
+            `corpus_file` arguments need to be passed (not both of them).
+        total_examples : int
+            Count of sentences.
+        total_words : int
+            Count of raw words in sentences.
+        epochs : int
             Number of iterations (epochs) over the corpus.
         start_alpha : float, optional
             Initial learning rate. If supplied, replaces the starting `alpha` from the constructor,
@@ -841,17 +893,19 @@ class Word2Vec(BaseWordEmbeddingsModel):
 
         Examples
         --------
-        >>> from gensim.models import Word2Vec
-        >>> sentences = [["cat", "say", "meow"], ["dog", "say", "woof"]]
-        >>>
-        >>> model = Word2Vec(min_count=1)
-        >>> model.build_vocab(sentences)  # prepare the model vocabulary
-        >>> model.train(sentences, total_examples=model.corpus_count, epochs=model.iter)  # train word vectors
-        (1, 30)
+        .. sourcecode:: pycon
+
+            >>> from gensim.models import Word2Vec
+            >>> sentences = [["cat", "say", "meow"], ["dog", "say", "woof"]]
+            >>>
+            >>> model = Word2Vec(min_count=1)
+            >>> model.build_vocab(sentences)  # prepare the model vocabulary
+            >>> model.train(sentences, total_examples=model.corpus_count, epochs=model.iter)  # train word vectors
+            (1, 30)
 
         """
         return super(Word2Vec, self).train(
-            sentences, total_examples=total_examples, total_words=total_words,
+            sentences=sentences, corpus_file=corpus_file, total_examples=total_examples, total_words=total_words,
             epochs=epochs, start_alpha=start_alpha, end_alpha=end_alpha, word_count=word_count,
             queue_factor=queue_factor, report_delay=report_delay, compute_loss=compute_loss, callbacks=callbacks)
 
@@ -934,7 +988,7 @@ class Word2Vec(BaseWordEmbeddingsModel):
         job_queue = Queue(maxsize=queue_factor * self.workers)
         progress_queue = Queue(maxsize=(queue_factor + 1) * self.workers)
 
-        workers = [threading.Thread(target=worker_loop) for _ in xrange(self.workers)]
+        workers = [threading.Thread(target=worker_loop) for _ in range(self.workers)]
         for thread in workers:
             thread.daemon = True  # make interrupting the process with ctrl+c easier
             thread.start()
@@ -961,7 +1015,7 @@ class Word2Vec(BaseWordEmbeddingsModel):
                 job_queue.put(items)
             except StopIteration:
                 logger.info("reached end of input; waiting to finish %i outstanding jobs", job_no - done_jobs + 1)
-                for _ in xrange(self.workers):
+                for _ in range(self.workers):
                     job_queue.put(None)  # give the workers heads up that they can finish -- no more work!
                 push_done = True
             try:
@@ -1031,7 +1085,7 @@ class Word2Vec(BaseWordEmbeddingsModel):
                 # TOCONSIDER: maybe mismatched vectors still useful enough to merge (truncating/padding)?
             if binary:
                 binary_len = dtype(REAL).itemsize * vector_size
-                for _ in xrange(vocab_size):
+                for _ in range(vocab_size):
                     # mixed text and binary: read text first, then binary
                     word = []
                     while True:
@@ -1357,10 +1411,12 @@ class LineSentence(object):
 
         Examples
         --------
-        >>> from gensim.test.utils import datapath
-        >>> sentences = LineSentence(datapath('lee_background.cor'))
-        >>> for sentence in sentences:
-        ...     pass
+        .. sourcecode:: pycon
+
+            >>> from gensim.test.utils import datapath
+            >>> sentences = LineSentence(datapath('lee_background.cor'))
+            >>> for sentence in sentences:
+            ...     pass
 
         """
         self.source = source
@@ -1446,6 +1502,40 @@ class PathLineSentences(object):
                         i += self.max_sentence_length
 
 
+def _scan_vocab_worker(stream, progress_queue, max_vocab_size=None, trim_rule=None):
+    """Do an initial scan of all words appearing in stream.
+
+    Note: This function can not be Word2VecVocab's method because
+    of multiprocessing synchronization specifics in Python.
+    """
+    min_reduce = 1
+    vocab = defaultdict(int)
+    checked_string_types = 0
+    sentence_no = -1
+    total_words = 0
+    for sentence_no, sentence in enumerate(stream):
+        if not checked_string_types:
+            if isinstance(sentence, string_types):
+                log_msg = "Each 'sentences' item should be a list of words (usually unicode strings). " \
+                          "First item here is instead plain %s." % type(sentence)
+                progress_queue.put(log_msg)
+
+            checked_string_types += 1
+
+        for word in sentence:
+            vocab[word] += 1
+
+        if max_vocab_size and len(vocab) > max_vocab_size:
+            utils.prune_vocab(vocab, min_reduce, trim_rule=trim_rule)
+            min_reduce += 1
+
+        total_words += len(sentence)
+
+    progress_queue.put((total_words, sentence_no + 1))
+    progress_queue.put(None)
+    return vocab
+
+
 class Word2VecVocab(utils.SaveLoad):
     """Vocabulary used by :class:`~gensim.models.word2vec.Word2Vec`."""
     def __init__(
@@ -1461,9 +1551,7 @@ class Word2VecVocab(utils.SaveLoad):
         self.max_final_vocab = max_final_vocab
         self.ns_exponent = ns_exponent
 
-    def scan_vocab(self, sentences, progress_per=10000, trim_rule=None):
-        """Do an initial scan of all words appearing in sentences."""
-        logger.info("collecting all words and their counts")
+    def _scan_vocab(self, sentences, progress_per, trim_rule):
         sentence_no = -1
         total_words = 0
         min_reduce = 1
@@ -1491,14 +1579,24 @@ class Word2VecVocab(utils.SaveLoad):
                 utils.prune_vocab(vocab, min_reduce, trim_rule=trim_rule)
                 min_reduce += 1
 
-        logger.info(
-            "collected %i word types from a corpus of %i raw words and %i sentences",
-            len(vocab), total_words, sentence_no + 1
-        )
         corpus_count = sentence_no + 1
         self.raw_vocab = vocab
         return total_words, corpus_count
 
+    def scan_vocab(self, sentences=None, corpus_file=None, progress_per=10000, workers=None, trim_rule=None):
+        logger.info("collecting all words and their counts")
+        if corpus_file:
+            sentences = LineSentence(corpus_file)
+
+        total_words, corpus_count = self._scan_vocab(sentences, progress_per, trim_rule)
+
+        logger.info(
+            "collected %i word types from a corpus of %i raw words and %i sentences",
+            len(self.raw_vocab), total_words, corpus_count
+        )
+
+        return total_words, corpus_count
+
     def sort_vocab(self, wv):
         """Sort the vocabulary so the most frequent words have the lowest indexes."""
         if len(wv.vectors):
@@ -1682,7 +1780,7 @@ class Word2VecVocab(utils.SaveLoad):
         # build the huffman tree
         heap = list(itervalues(wv.vocab))
         heapq.heapify(heap)
-        for i in xrange(len(wv.vocab) - 1):
+        for i in range(len(wv.vocab) - 1):
             min1, min2 = heapq.heappop(heap), heapq.heappop(heap)
             heapq.heappush(
                 heap, Vocab(count=min1.count + min2.count, index=i + len(wv.vocab), left=min1, right=min2)
@@ -1720,10 +1818,10 @@ class Word2VecVocab(utils.SaveLoad):
         self.cum_table = zeros(vocab_size, dtype=uint32)
         # compute sum of all power (Z in paper)
         train_words_pow = 0.0
-        for word_index in xrange(vocab_size):
+        for word_index in range(vocab_size):
             train_words_pow += wv.vocab[wv.index2word[word_index]].count**self.ns_exponent
         cumulative = 0.0
-        for word_index in xrange(vocab_size):
+        for word_index in range(vocab_size):
             cumulative += wv.vocab[wv.index2word[word_index]].count**self.ns_exponent
             self.cum_table[word_index] = round(cumulative / train_words_pow * domain)
         if len(self.cum_table) > 0:
@@ -1756,7 +1854,7 @@ class Word2VecTrainables(utils.SaveLoad):
         logger.info("resetting layer weights")
         wv.vectors = empty((len(wv.vocab), wv.vector_size), dtype=REAL)
         # randomize weights vector by vector, rather than materializing a huge random matrix in RAM at once
-        for i in xrange(len(wv.vocab)):
+        for i in range(len(wv.vocab)):
             # construct deterministic seed from word AND seed argument
             wv.vectors[i] = self.seeded_vector(wv.index2word[i] + str(self.seed), wv.vector_size)
         if hs:
@@ -1774,7 +1872,7 @@ class Word2VecTrainables(utils.SaveLoad):
         newvectors = empty((gained_vocab, wv.vector_size), dtype=REAL)
 
         # randomize the remaining words
-        for i in xrange(len(wv.vectors), len(wv.vocab)):
+        for i in range(len(wv.vectors), len(wv.vocab)):
             # construct deterministic seed from word AND seed argument
             newvectors[i - len(wv.vectors)] = self.seeded_vector(wv.index2word[i] + str(self.seed), wv.vector_size)
 
@@ -1790,7 +1888,8 @@ class Word2VecTrainables(utils.SaveLoad):
         if hs:
             self.syn1 = vstack([self.syn1, zeros((gained_vocab, self.layer1_size), dtype=REAL)])
         if negative:
-            self.syn1neg = vstack([self.syn1neg, zeros((gained_vocab, self.layer1_size), dtype=REAL)])
+            pad = zeros((gained_vocab, self.layer1_size), dtype=REAL)
+            self.syn1neg = vstack([self.syn1neg, pad])
         wv.vectors_norm = None
 
         # do not suppress learning for already learned words
