@@ -25,24 +25,29 @@ support adding new document to the index dynamically.
 
 Once the index has been initialized, you can query for document similarity simply by
 
->>> from gensim.test.utils import common_corpus, common_dictionary, get_tmpfile
->>>
->>> index_tmpfile = get_tmpfile("index")
->>> query = [(1, 2), (6, 1), (7, 2)]
->>>
->>> index = Similarity(index_tmpfile, common_corpus, num_features=len(common_dictionary)) # build the index
->>> similarities = index[query] # get similarities between the query and all index documents
+.. sourcecode:: pycon
+
+    >>> from gensim.test.utils import common_corpus, common_dictionary, get_tmpfile
+    >>>
+    >>> index_tmpfile = get_tmpfile("index")
+    >>> query = [(1, 2), (6, 1), (7, 2)]
+    >>>
+    >>> index = Similarity(index_tmpfile, common_corpus, num_features=len(common_dictionary))  # build the index
+    >>> similarities = index[query]  # get similarities between the query and all index documents
 
 If you have more query documents, you can submit them all at once, in a batch
 
->>> from gensim.test.utils import common_corpus, common_dictionary, get_tmpfile
->>>
->>> index_tmpfile = get_tmpfile("index")
->>> batch_of_documents = common_corpus[:]  # only as example
->>> index = Similarity(index_tmpfile, common_corpus, num_features=len(common_dictionary)) # build the index
->>>
->>> for similarities in index[batch_of_documents]: # the batch is simply an iterable of documents, aka gensim corpus.
-...     pass
+.. sourcecode:: pycon
+
+    >>> from gensim.test.utils import common_corpus, common_dictionary, get_tmpfile
+    >>>
+    >>> index_tmpfile = get_tmpfile("index")
+    >>> batch_of_documents = common_corpus[:]  # only as example
+    >>> index = Similarity(index_tmpfile, common_corpus, num_features=len(common_dictionary))  # build the index
+    >>>
+    >>> # the batch is simply an iterable of documents, aka gensim corpus:
+    >>> for similarities in index[batch_of_documents]:
+    ...     pass
 
 The benefit of this batch (aka "chunked") querying is a much better performance.
 To see the speed-up on your machine, run ``python -m gensim.test.simspeed``
@@ -52,16 +57,17 @@ There is also a special syntax for when you need similarity of documents in the
 to the index itself (i.e. queries = the indexed documents themselves). This special syntax
 uses the faster, batch queries internally and **is ideal for all-vs-all pairwise similarities**:
 
->>> from gensim.test.utils import common_corpus, common_dictionary, get_tmpfile
->>>
->>> index_tmpfile = get_tmpfile("index")
->>> index = Similarity(index_tmpfile, common_corpus, num_features=len(common_dictionary)) # build the index
->>>
->>> for similarities in index: # yield similarities of the 1st indexed document, then 2nd...
-...     pass
+.. sourcecode:: pycon
+
+    >>> from gensim.test.utils import common_corpus, common_dictionary, get_tmpfile
+    >>>
+    >>> index_tmpfile = get_tmpfile("index")
+    >>> index = Similarity(index_tmpfile, common_corpus, num_features=len(common_dictionary))  # build the index
+    >>>
+    >>> for similarities in index:  # yield similarities of the 1st indexed document, then 2nd...
+    ...     pass
 
 """
-
 import logging
 import itertools
 import os
@@ -71,7 +77,8 @@ import numpy
 import scipy.sparse
 
 from gensim import interfaces, utils, matutils
-from six.moves import map as imap, xrange, zip as izip
+from .termsim import SparseTermSimilarityMatrix
+from six.moves import map, range, zip
 
 
 logger = logging.getLogger(__name__)
@@ -238,25 +245,27 @@ class Similarity(interfaces.SimilarityABC):
 
     Examples
     --------
-    >>> from gensim.corpora.textcorpus import TextCorpus
-    >>> from gensim.test.utils import datapath, get_tmpfile
-    >>> from gensim.similarities import Similarity
-    >>>
-    >>> corpus = TextCorpus(datapath('testcorpus.mm'))
-    >>> index_temp = get_tmpfile("index")
-    >>> index = Similarity(index_temp, corpus, num_features=400)  # create index
-    >>>
-    >>> query = next(iter(corpus))
-    >>> result = index[query]  # search similar to `query` in index
-    >>>
-    >>> for sims in index[corpus]: # if you have more query documents, you can submit them all at once, in a batch
-    ...     pass
-    >>>
-    >>> # There is also a special syntax for when you need similarity of documents in the index
-    >>> # to the index itself (i.e. queries=indexed documents themselves). This special syntax
-    >>> # uses the faster, batch queries internally and **is ideal for all-vs-all pairwise similarities**:
-    >>> for similarities in index: # yield similarities of the 1st indexed document, then 2nd...
-    ...     pass
+    .. sourcecode:: pycon
+
+        >>> from gensim.corpora.textcorpus import TextCorpus
+        >>> from gensim.test.utils import datapath, get_tmpfile
+        >>> from gensim.similarities import Similarity
+        >>>
+        >>> corpus = TextCorpus(datapath('testcorpus.mm'))
+        >>> index_temp = get_tmpfile("index")
+        >>> index = Similarity(index_temp, corpus, num_features=400)  # create index
+        >>>
+        >>> query = next(iter(corpus))
+        >>> result = index[query]  # search similar to `query` in index
+        >>>
+        >>> for sims in index[corpus]:  # if you have more query documents, you can submit them all at once, in a batch
+        ...     pass
+        >>>
+        >>> # There is also a special syntax for when you need similarity of documents in the index
+        >>> # to the index itself (i.e. queries=indexed documents themselves). This special syntax
+        >>> # uses the faster, batch queries internally and **is ideal for all-vs-all pairwise similarities**:
+        >>> for similarities in index:  # yield similarities of the 1st indexed document, then 2nd...
+        ...     pass
 
     See Also
     --------
@@ -264,8 +273,6 @@ class Similarity(interfaces.SimilarityABC):
         Index similarity (dense with cosine distance).
     :class:`~gensim.similarities.docsim.SparseMatrixSimilarity`
         Index similarity (sparse with cosine distance).
-    :class:`~gensim.similarities.docsim.SoftCosineSimilarity`
-        Index similarity (with soft-cosine distance).
     :class:`~gensim.similarities.docsim.WmdSimilarity`
         Index similarity (with word-mover distance).
 
@@ -326,7 +333,7 @@ class Similarity(interfaces.SimilarityABC):
 
     def __len__(self):
         """Get length of index."""
-        return len(self.fresh_docs) + sum([len(shard) for shard in self.shards])
+        return len(self.fresh_docs) + sum(len(shard) for shard in self.shards)
 
     def __str__(self):
         return "Similarity index with %i documents in %i shards (stored under %s)" % (
@@ -348,16 +355,18 @@ class Similarity(interfaces.SimilarityABC):
 
         Examples
         --------
-        >>> from gensim.corpora.textcorpus import TextCorpus
-        >>> from gensim.test.utils import datapath, get_tmpfile
-        >>> from gensim.similarities import Similarity
-        >>>
-        >>> corpus = TextCorpus(datapath('testcorpus.mm'))
-        >>> index_temp = get_tmpfile("index")
-        >>> index = Similarity(index_temp, corpus, num_features=400)  # create index
-        >>>
-        >>> one_more_corpus = TextCorpus(datapath('testcorpus.txt'))
-        >>> index.add_documents(one_more_corpus)  # add more documents in corpus
+        .. sourcecode:: pycon
+
+            >>> from gensim.corpora.textcorpus import TextCorpus
+            >>> from gensim.test.utils import datapath, get_tmpfile
+            >>> from gensim.similarities import Similarity
+            >>>
+            >>> corpus = TextCorpus(datapath('testcorpus.mm'))
+            >>> index_temp = get_tmpfile("index")
+            >>> index = Similarity(index_temp, corpus, num_features=400)  # create index
+            >>>
+            >>> one_more_corpus = TextCorpus(datapath('testcorpus.txt'))
+            >>> index.add_documents(one_more_corpus)  # add more documents in corpus
 
         """
         min_ratio = 1.0  # 0.5 to only reopen shards that are <50% complete
@@ -462,11 +471,11 @@ class Similarity(interfaces.SimilarityABC):
         if PARALLEL_SHARDS and PARALLEL_SHARDS > 1:
             logger.debug("spawning %i query processes", PARALLEL_SHARDS)
             pool = multiprocessing.Pool(PARALLEL_SHARDS)
-            result = pool.imap(query_shard, args, chunksize=1 + len(list(args)) / PARALLEL_SHARDS)
+            result = pool.imap(query_shard, args, chunksize=1 + len(self.shards) / PARALLEL_SHARDS)
         else:
             # serial processing, one shard after another
             pool = None
-            result = imap(query_shard, args)
+            result = map(query_shard, args)
         return pool, result
 
     def __getitem__(self, query):
@@ -490,14 +499,15 @@ class Similarity(interfaces.SimilarityABC):
 
         Examples
         --------
-        >>> from gensim.corpora.textcorpus import TextCorpus
-        >>> from gensim.test.utils import datapath
-        >>> from gensim.similarities import Similarity
-        >>> import gensim.downloader as api
-        >>>
-        >>> corpus = TextCorpus(datapath('testcorpus.txt'))
-        >>> index = Similarity('temp', corpus, num_features=400)
-        >>> result = index[corpus]  # pairwise similarities of each document against each document
+        .. sourcecode:: pycon
+
+            >>> from gensim.corpora.textcorpus import TextCorpus
+            >>> from gensim.test.utils import datapath
+            >>> from gensim.similarities import Similarity
+            >>>
+            >>> corpus = TextCorpus(datapath('testcorpus.txt'))
+            >>> index = Similarity('temp', corpus, num_features=400)
+            >>> result = index[corpus]  # pairwise similarities of each document against each document
 
         """
         self.close_shard()  # no-op if no documents added to index since last query
@@ -537,7 +547,7 @@ class Similarity(interfaces.SimilarityABC):
                     shard_result = [convert(shard_no, doc) for doc in result]
                     results.append(shard_result)
                 result = []
-                for parts in izip(*results):
+                for parts in zip(*results):
                     merged = heapq.nlargest(self.num_best, itertools.chain(*parts), key=lambda item: item[1])
                     result.append(merged)
         if pool:
@@ -562,15 +572,16 @@ class Similarity(interfaces.SimilarityABC):
 
         Examples
         --------
-        >>> from gensim.corpora.textcorpus import TextCorpus
-        >>> from gensim.test.utils import datapath
-        >>> from gensim.similarities import Similarity
-        >>> import gensim.downloader as api
-        >>>
-        >>> # Create index:
-        >>> corpus = TextCorpus(datapath('testcorpus.txt'))
-        >>> index = Similarity('temp', corpus, num_features=400)
-        >>> vector = index.vector_by_id(1)
+        .. sourcecode:: pycon
+
+            >>> from gensim.corpora.textcorpus import TextCorpus
+            >>> from gensim.test.utils import datapath
+            >>> from gensim.similarities import Similarity
+            >>>
+            >>> # Create index:
+            >>> corpus = TextCorpus(datapath('testcorpus.txt'))
+            >>> index = Similarity('temp', corpus, num_features=400)
+            >>> vector = index.vector_by_id(1)
 
         """
         self.close_shard()  # no-op if no documents added to index since last query
@@ -599,13 +610,15 @@ class Similarity(interfaces.SimilarityABC):
 
         Examples
         --------
-        >>> from gensim.corpora.textcorpus import TextCorpus
-        >>> from gensim.test.utils import datapath
-        >>> from gensim.similarities import Similarity
-        >>>
-        >>> corpus = TextCorpus(datapath('testcorpus.txt'))
-        >>> index = Similarity('temp', corpus, num_features=400)
-        >>> similarities = index.similarity_by_id(1)
+        .. sourcecode:: pycon
+
+            >>> from gensim.corpora.textcorpus import TextCorpus
+            >>> from gensim.test.utils import datapath
+            >>> from gensim.similarities import Similarity
+            >>>
+            >>> corpus = TextCorpus(datapath('testcorpus.txt'))
+            >>> index = Similarity('temp', corpus, num_features=400)
+            >>> similarities = index.similarity_by_id(1)
 
         """
         query = self.vector_by_id(docpos)
@@ -659,7 +672,7 @@ class Similarity(interfaces.SimilarityABC):
 
         for shard in self.shards:
             query = shard.get_index().index
-            for chunk_start in xrange(0, query.shape[0], chunksize):
+            for chunk_start in range(0, query.shape[0], chunksize):
                 # scipy.sparse doesn't allow slicing beyond real size of the matrix
                 # (unlike numpy). so, clip the end of the chunk explicitly to make
                 # scipy.sparse happy
@@ -692,18 +705,20 @@ class Similarity(interfaces.SimilarityABC):
 
         Examples
         --------
-        >>> from gensim.corpora.textcorpus import TextCorpus
-        >>> from gensim.test.utils import datapath, get_tmpfile
-        >>> from gensim.similarities import Similarity
-        >>>
-        >>> temp_fname = get_tmpfile("index")
-        >>> output_fname = get_tmpfile("saved_index")
-        >>>
-        >>> corpus = TextCorpus(datapath('testcorpus.txt'))
-        >>> index = Similarity(output_fname, corpus, num_features=400)
-        >>>
-        >>> index.save(output_fname)
-        >>> loaded_index = index.load(output_fname)
+        .. sourcecode:: pycon
+
+            >>> from gensim.corpora.textcorpus import TextCorpus
+            >>> from gensim.test.utils import datapath, get_tmpfile
+            >>> from gensim.similarities import Similarity
+            >>>
+            >>> temp_fname = get_tmpfile("index")
+            >>> output_fname = get_tmpfile("saved_index")
+            >>>
+            >>> corpus = TextCorpus(datapath('testcorpus.txt'))
+            >>> index = Similarity(output_fname, corpus, num_features=400)
+            >>>
+            >>> index.save(output_fname)
+            >>> loaded_index = index.load(output_fname)
 
         """
         self.close_shard()
@@ -726,12 +741,14 @@ class MatrixSimilarity(interfaces.SimilarityABC):
 
     Examples
     --------
-    >>> from gensim.test.utils import common_corpus, common_dictionary
-    >>> from gensim.similarities import MatrixSimilarity
-    >>>
-    >>> query = [(1, 2), (5, 4)]
-    >>> index = MatrixSimilarity(common_corpus, num_features=len(common_dictionary))
-    >>> sims = index[query]
+    .. sourcecode:: pycon
+
+        >>> from gensim.test.utils import common_corpus, common_dictionary
+        >>> from gensim.similarities import MatrixSimilarity
+        >>>
+        >>> query = [(1, 2), (5, 4)]
+        >>> index = MatrixSimilarity(common_corpus, num_features=len(common_dictionary))
+        >>> sims = index[query]
 
     """
     def __init__(self, corpus, num_best=None, dtype=numpy.float32, num_features=None, chunksize=256, corpus_len=None):
@@ -805,7 +822,7 @@ class MatrixSimilarity(interfaces.SimilarityABC):
 
         Parameters
         ----------
-        query : {list of (int, number), iterable of list of (int, number), :class:`scipy.sparse.csr_matrix`
+        query : {list of (int, number), iterable of list of (int, number), :class:`scipy.sparse.csr_matrix`}
             Document or collection of documents.
 
         Return
@@ -844,22 +861,22 @@ class SoftCosineSimilarity(interfaces.SimilarityABC):
 
     Examples
     --------
-    >>> from gensim.test.utils import common_texts
-    >>> from gensim.corpora import Dictionary
-    >>> from gensim.models import Word2Vec
-    >>> from gensim.similarities import SoftCosineSimilarity
-    >>>
-    >>> model = Word2Vec(common_texts, size=20, min_count=1)  # train word-vectors
-    >>> dictionary = Dictionary(common_texts)
-    >>> bow_corpus = [dictionary.doc2bow(document) for document in common_texts]
-    >>>
-    >>> similarity_matrix = model.wv.similarity_matrix(dictionary)  # construct similarity matrix
-    >>> index = SoftCosineSimilarity(bow_corpus, similarity_matrix, num_best=10)
-    >>>
-    >>> # Make a query.
-    >>> query = 'graph trees computer'.split()
-    >>> # calculate similarity between query and each doc from bow_corpus
-    >>> sims = index[dictionary.doc2bow(query)]
+    .. sourcecode:: pycon
+
+        >>> from gensim.test.utils import common_texts
+        >>> from gensim.corpora import Dictionary
+        >>> from gensim.models import Word2Vec, WordEmbeddingSimilarityIndex
+        >>> from gensim.similarities import SoftCosineSimilarity, TermSimilarityMatrix
+        >>>
+        >>> model = Word2Vec(common_texts, size=20, min_count=1)  # train word-vectors
+        >>> termsim_index = WordEmbeddingSimilarityIndex(model)
+        >>> dictionary = Dictionary(common_texts)
+        >>> bow_corpus = [dictionary.doc2bow(document) for document in common_texts]
+        >>> similarity_matrix = TermSimilarityMatrix(termsim_index, dictionary)  # construct similarity matrix
+        >>> docsim_index = SoftCosineSimilarity(bow_corpus, similarity_matrix, num_best=10)
+        >>>
+        >>> query = 'graph trees computer'.split()  # make a query
+        >>> sims = docsim_index[dictionary.doc2bow(query)]  # calculate similarity of query to each doc from bow_corpus
 
     Check out `Tutorial Notebook
     <https://github.com/RaRe-Technologies/gensim/blob/develop/docs/notebooks/soft_cosine_tutorial.ipynb>`_
@@ -873,9 +890,8 @@ class SoftCosineSimilarity(interfaces.SimilarityABC):
         ----------
         corpus: iterable of list of (int, float)
             A list of documents in the BoW format.
-        similarity_matrix : :class:`scipy.sparse.csc_matrix`
-            A term similarity matrix, typically produced by
-            :meth:`~gensim.models.keyedvectors.WordEmbeddingsKeyedVectors.similarity_matrix`.
+        similarity_matrix : :class:`gensim.similarities.SparseTermSimilarityMatrix`
+            A term similarity matrix.
         num_best : int, optional
             The number of results to retrieve for a query, if None - return similarities with all elements from corpus.
         chunksize: int, optional
@@ -883,14 +899,23 @@ class SoftCosineSimilarity(interfaces.SimilarityABC):
 
         See Also
         --------
-        :meth:`gensim.models.keyedvectors.WordEmbeddingsKeyedVectors.similarity_matrix`
-            A term similarity matrix produced from term embeddings.
-        :func:`gensim.matutils.softcossim`
-            The Soft Cosine Measure.
+        :class:`gensim.similarities.SparseTermSimilarityMatrix`
+            A sparse term similarity matrix build using a term similarity index.
+        :class:`gensim.similarities.LevenshteinSimilarityIndex`
+            A term similarity index that computes Levenshtein similarities between terms.
+        :class:`gensim.models.WordEmbeddingSimilarityIndex`
+            A term similarity index that computes cosine similarities between word embeddings.
 
         """
+        if scipy.sparse.issparse(similarity_matrix):
+            logger.warn(
+                "Support for passing an unencapsulated sparse matrix will be removed in 4.0.0, pass "
+                "a SparseTermSimilarityMatrix instance instead")
+            self.similarity_matrix = SparseTermSimilarityMatrix(similarity_matrix)
+        else:
+            self.similarity_matrix = similarity_matrix
+
         self.corpus = corpus
-        self.similarity_matrix = similarity_matrix
         self.num_best = num_best
         self.chunksize = chunksize
 
@@ -914,7 +939,7 @@ class SoftCosineSimilarity(interfaces.SimilarityABC):
 
         Parameters
         ----------
-        query : {list of (int, number), iterable of list of (int, number)
+        query : {list of (int, number), iterable of list of (int, number)}
             Document or collection of documents.
 
         Return
@@ -923,38 +948,26 @@ class SoftCosineSimilarity(interfaces.SimilarityABC):
             Similarity matrix.
 
         """
+        if not self.corpus:
+            return numpy.array()
 
         is_corpus, query = utils.is_corpus(query)
-        if not is_corpus:
-            if isinstance(query, numpy.ndarray):
-                # Convert document indexes to actual documents.
-                query = [self.corpus[i] for i in query]
-            else:
-                query = [query]
+        if not is_corpus and isinstance(query, numpy.ndarray):
+            query = [self.corpus[i] for i in query]  # convert document indexes to actual documents
+        result = self.similarity_matrix.inner_product(query, self.corpus, normalized=True)
 
-        result = []
-        for query_document in query:
-            # Compute similarity for each query.
-            qresult = [matutils.softcossim(query_document, corpus_document, self.similarity_matrix)
-                       for corpus_document in self.corpus]
-            qresult = numpy.array(qresult)
-
-            # Append single query result to list of all results.
-            result.append(qresult)
-
-        if is_corpus:
-            result = numpy.array(result)
-        else:
-            result = result[0]
-
-        return result
+        if scipy.sparse.issparse(result):
+            return numpy.asarray(result.todense())
+        if numpy.isscalar(result):
+            return numpy.array(result)
+        return numpy.asarray(result)[0]
 
     def __str__(self):
         return "%s<%i docs, %i features>" % (self.__class__.__name__, len(self), self.similarity_matrix.shape[0])
 
 
 class WmdSimilarity(interfaces.SimilarityABC):
-    """Compute negative WMD similarity against a corpus of documents by storing the index matrix in memory.
+    """Compute negative WMD similarity against a corpus of documents.
 
     See :class:`~gensim.models.keyedvectors.WordEmbeddingsKeyedVectors` for more information.
     Also, tutorial `notebook
@@ -971,19 +984,18 @@ class WmdSimilarity(interfaces.SimilarityABC):
 
     Example
     -------
-    >>> from gensim.test.utils import common_texts
-    >>> from gensim.corpora import Dictionary
-    >>> from gensim.models import Word2Vec
-    >>> from gensim.similarities import WmdSimilarity
-    >>>
-    >>> model = Word2Vec(common_texts, size=20, min_count=1)  # train word-vectors
-    >>> dictionary = Dictionary(common_texts)
-    >>> bow_corpus = [dictionary.doc2bow(document) for document in common_texts]
-    >>>
-    >>> index = WmdSimilarity(bow_corpus, model)
-    >>> # Make query.
-    >>> query = 'trees'
-    >>> sims = index[query]
+    .. sourcecode:: pycon
+
+        >>> from gensim.test.utils import common_texts
+        >>> from gensim.models import Word2Vec
+        >>> from gensim.similarities import WmdSimilarity
+        >>>
+        >>> model = Word2Vec(common_texts, size=20, min_count=1)  # train word-vectors
+        >>>
+        >>> index = WmdSimilarity(common_texts, model)
+        >>> # Make query.
+        >>> query = ['trees']
+        >>> sims = index[query]
 
     """
     def __init__(self, corpus, w2v_model, num_best=None, normalize_w2v_and_replace=True, chunksize=256):
@@ -991,8 +1003,8 @@ class WmdSimilarity(interfaces.SimilarityABC):
 
         Parameters
         ----------
-        corpus: iterable of list of (int, float)
-            A list of documents in the BoW format.
+        corpus: iterable of list of str
+            A list of documents, each of which is a list of tokens.
         w2v_model: :class:`~gensim.models.word2vec.Word2VecTrainables`
             A trained word2vec model.
         num_best: int, optional
@@ -1031,7 +1043,7 @@ class WmdSimilarity(interfaces.SimilarityABC):
 
         Parameters
         ----------
-        query : {list of (int, number), iterable of list of (int, number)
+        query : {list of str, iterable of list of str}
             Document or collection of documents.
 
         Return
@@ -1051,7 +1063,7 @@ class WmdSimilarity(interfaces.SimilarityABC):
         result = []
         for qidx in range(n_queries):
             # Compute similarity for each query.
-            qresult = [self.w2v_model.wmdistance(document, query[qidx]) for document in self.corpus]
+            qresult = [self.w2v_model.wv.wmdistance(document, query[qidx]) for document in self.corpus]
             qresult = numpy.array(qresult)
             qresult = 1. / (1. + qresult)  # Similarity is the negative of the distance.
 
@@ -1167,7 +1179,7 @@ class SparseMatrixSimilarity(interfaces.SimilarityABC):
 
         Parameters
         ----------
-        query : {list of (int, number), iterable of list of (int, number), :class:`scipy.sparse.csr_matrix`
+        query : {list of (int, number), iterable of list of (int, number), :class:`scipy.sparse.csr_matrix`}
             Document or collection of documents.
 
         Return
